@@ -231,6 +231,10 @@ unknown_proc_msg((:-)/2) -->
     !,
     [nl, '  Rules must be loaded from a file'],
     faq('ToplevelMode').
+unknown_proc_msg((=>)/2) -->
+    !,
+    [nl, '  Rules must be loaded from a file'],
+    faq('ToplevelMode').
 unknown_proc_msg((:-)/1) -->
     !,
     [nl, '  Directives must be loaded from a file'],
@@ -461,6 +465,10 @@ swi_message(determinism_error(PI, det, fail, guard)) -->
     [ 'Procedure ~p failed after $-guard'-[PI] ].
 swi_message(determinism_error(PI, det, fail, guard_in_caller)) -->
     [ 'Procedure ~p failed after $-guard in caller'-[PI] ].
+swi_message(determinism_error(Goal, det, fail, goal)) -->
+    [ 'Goal ~p failed'-[Goal] ].
+swi_message(determinism_error(Goal, det, nondet, goal)) -->
+    [ 'Goal ~p succeeded with a choice point'-[Goal] ].
 swi_message(qlf_format_error(File, Message)) -->
     [ '~w: Invalid QLF file: ~w'-[File, Message] ].
 
