@@ -374,9 +374,11 @@ translate([\(Section, -, [{Title}])|T], Mode, Mode, HTML) :-
     append(TitleHtml, BodyHtml, HTML),
     translate(T, Mode, Mode, BodyHtml).
 translate(['`','`'|T0], Mode0, Mode, [html(' &ldquo;')|T]) :-
+    \+ in_macro(code(_)),
     !,
     translate(T0, Mode0, Mode, T).
 translate(['\'','\''|T0], Mode0, Mode, [html('&rdquo; ')|T]) :-
+    \+ in_macro(code(_)),
     !,
     translate(T0, Mode0, Mode, T).
 translate([H0|T0], Mode0, Mode, [H|T]) :-
@@ -725,13 +727,31 @@ expand_macro(#Macro, HTML) :-
     tex_extension_module(M),
     M:'#'(Macro, HTML0),
     !,
-    expand_macros(HTML0, HTML).
+    push_macro(Macro),
+    expand_macros(HTML0, HTML),
+    pop_macro.
 expand_macro(List, Expanded) :-
     List = [_|_],
     !,
     expand_macros(List, Expanded).
 expand_macro(NoExpand, NoExpand).
 
+push_macro(Macro) :-
+    nb_current(macro_stack, Stack),
+    !,
+    b_setval(macro_stack, [Macro|Stack]).
+push_macro(Macro) :-
+    b_setval(macro_stack, [Macro]).
+
+pop_macro :-
+    nb_current(macro_stack, [_|Stack]),
+    !,
+    b_setval(macro_stack, Stack).
+
+in_macro(Env) :-
+    nb_current(macro_stack, Stack),
+    \+ \+ memberchk(Env, Stack).
+
 
                  /*******************************
                  *             CITES            *
@@ -858,6 +878,8 @@ env(titlepage(_, _Page), []) :-
     ).
 env(tabular([{Format}], Tokens), HTML) :-
     translate_table(Format, Tokens, HTML).
+env(tabulary([_Width, {Format}], Tokens), HTML) :-
+    translate_table(Format, Tokens, HTML).
 env(array([{Format}], Tokens), HTML) :-
     translate_table(Format, Tokens, HTML).
 env(table(_, Tokens), Table) :-
@@ -2008,7 +2030,7 @@ index_terms([Term0|T0], [ html('<dt>'),
                        | TH
                        ]) :-
     findall(Tag, index(_, Term0, Tag), Tags),
-    tex_atom_to_tokens(Term0, Tokens),
+    term_tokens(Term0, Tokens),
     translate(Tokens, index, _, Term),
     (   member(+PrimeTag, Tags)
     ->  HtmlTerm = #lref(idx, PrimeTag, Term)
@@ -2017,6 +2039,14 @@ index_terms([Term0|T0], [ html('<dt>'),
     maplist(index_href, Tags, Where),
     index_terms(T0, TH).
 
+term_tokens('$/0', [verb(!,'$/0')]) :-
+    !.
+term_tokens('$/1', [verb(!,'$/1')]) :-
+    !.
+term_tokens(Term, Tokens) :-
+    tex_atom_to_tokens(Term, Tokens).
+
+
 index_href(+(_), []) :- !.
 index_href(Tag:Label, [' ', #lref(idx, Label, Tag)]) :- !.
 index_href(Tag, [' ', #lref(sec, RefName, Tag)]) :-
@@ -2266,12 +2296,21 @@ table_columns([],      Ncols, Ncols, []).
 table_columns([0'l|T], NC0,   NC,    [[]|TH]) :-
     NC1 is NC0 + 1,
     table_columns(T, NC1, NC, TH).
+table_columns([0'L|T], NC0,   NC,    [[]|TH]) :-
+    NC1 is NC0 + 1,
+    table_columns(T, NC1, NC, TH).
 table_columns([0'c|T], NC0,   NC,    [[align=center]|TH]) :-
     NC1 is NC0 + 1,
     table_columns(T, NC1, NC, TH).
+table_columns([0'C|T], NC0,   NC,    [[align=center]|TH]) :-
+    NC1 is NC0 + 1,
+    table_columns(T, NC1, NC, TH).
 table_columns([0'r|T], NC0,   NC,    [[align=right]|TH]) :-
     NC1 is NC0 + 1,
     table_columns(T, NC1, NC, TH).
+table_columns([0'R|T], NC0,   NC,    [[align=right]|TH]) :-
+    NC1 is NC0 + 1,
+    table_columns(T, NC1, NC, TH).
 table_columns([0'p|T0], NC0,   NC,   [Col|TH]) :-
     phrase(parbox_width(_W), T0, T),
     Col = [],
