@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2006-2018, University of Amsterdam
+    Copyright (c)  2006-2020, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
     All rights reserved.
@@ -46,7 +46,8 @@
 
 :- autoload(doc_html,
 	    [ object_tree/5, private/2, object_page_header/4, objects/4,
-	      object_href/2, object_synopsis/4, object_page_footer/4,
+	      object_href/2, object_synopsis/4, object_footer/4,
+              object_page_footer/4,
 	      object_ref/4, object_page/4,
               object_source_button//2
 	    ]).
@@ -54,7 +55,7 @@
 :- autoload(doc_search,[search_form/3]).
 :- autoload(doc_util,[atom_to_object/2,atom_pi/2]).
 :- autoload(man_index,[manual_object/5]).
-:- autoload(library(apply),[maplist/2,maplist/3]).
+:- autoload(library(apply),[maplist/2,maplist/3,convlist/3]).
 :- autoload(library(debug),[assertion/1,debug/3]).
 :- autoload(library(error),[permission_error/3,existence_error/2]).
 :- autoload(library(filesex),
@@ -746,15 +747,17 @@ man_match(packages, packages, _) -->
 
           <p>
           See also <a href="/pack/list">Add-ons</a> for extensions
-          provided by the community that must be installed seperately
+          provided by the community that must be installed separately
           using
           <a href="/pldoc/doc_for?object=pack_install/1">pack_install/1</a>.</p>
          |}).
 man_match(root, root, _) -->
     !,
     man_overview([]).
-man_match((Parent+Path)-(Obj+[element(dt,A,C0)|DD]), Obj, Options) -->
+man_match((Parent+Path)-(Obj+DOM), Obj, Options) -->
     { \+ option(synopsis(false), Options),
+      DOM = [element(dt,A,C0)|DD],
+      convlist(dt_obj, DOM, Objs),
       option(link_source(Link), Options, true),
       man_qualified_object(Obj, Parent, LibOpt, QObj, Section),
       !,
@@ -766,10 +769,14 @@ man_match((Parent+Path)-(Obj+[element(dt,A,C0)|DD]), Obj, Options) -->
     dom_list([ element(dt,[],[\man_synopsis(QObj, Section, LibOpt)]),
                element(dt,A,C)
              | DD
-             ], Path, Options).
+             ], Path, Options),
+    object_footer(Objs, Options).
 man_match((_Parent+Path)-(Obj+DOM), Obj, Options) -->
     dom_list(DOM, Path, Options).
 
+dt_obj(element(dt,_,C), Obj) :-
+    xpath(C, //a(@id=Atom), _),
+    atom_to_object(Atom, Obj).
 
 :- html_meta
     dom_list(html, +, +, ?, ?).
@@ -800,11 +807,14 @@ dom_element(a, Att, Content, Path, Options) -->
     },
     !,
     html(a(href(Myref), \dom_list(Content, Path, Options))).
-dom_element(span, Att, [CDATA], _, _Options) -->
+dom_element(span, Att, [CDATA], _, Options) -->
     { memberchk(class='pred-ext', Att),
       atom_pi(CDATA, PI),
       documented(PI),
-      http_link_to_id(pldoc_man, [predicate=CDATA], HREF)
+      (   option(server(false), Options)
+      ->  public_link(predicate(CDATA), HREF)
+      ;   http_link_to_id(pldoc_man, [predicate=CDATA], HREF)
+      )
     },
     !,
     html(a(href(HREF), CDATA)).
@@ -855,6 +865,19 @@ ensure_slash(Dir, DirS) :-
     ;   atom_concat(Dir, /, DirS)
     ).
 
+%!  public_link(+Spec, -HREF)
+%
+%   We do not have a web server.  Create   a  link  to the public server
+%   instead.
+%
+%   @bug The predicate may not be there.
+
+public_link(predicate(CDATA), HREF) :-
+    uri_encoded(query_value, CDATA, Encoded),
+    atom_concat('https://www.swi-prolog.org/pldoc/doc_for?object=',
+                Encoded, HREF).
+
+
 %!  documented(+PI) is semidet.
 %
 %   True if we have documentation about PI
@@ -880,14 +903,18 @@ documented(PI) :-
 %       $ File#flag:Name :
 %       Rewrite to =|section(Level, NT, ID, FilePath)#flag:Name|=
 %
+%       $ File#gloss:Name :
+%       Rewrite to =|section(Level, NT, ID, FilePath)#gloss:Name|=
+%
 %       $ File#Name()
 %       Rewrite to /man/CAPI=Name
 %
 %   @param Class    Class of the <A>.  Supported classes are
 %
-%           | sec  | Link to a section     |
-%           | pred | Link to a predicate   |
-%           | flag | link to a Prolog flag |
+%           | sec   | Link to a section     |
+%           | pred  | Link to a predicate   |
+%           | flag  | Link to a Prolog flag |
+%           | gloss | Link to a glossary    |
 %
 %   @param Ref0     Initial reference from the =a= element
 %   @param Path     Currently loaded file
@@ -963,6 +990,18 @@ rewrite_ref(flag, Ref0, Path, Ref) :-
     !,
     object_href(Obj, Ref1),
     format(string(Ref), '~w#~w', [Ref1, Fragment]).
+rewrite_ref(gloss, Ref0, Path, Ref) :-
+    sub_atom(Ref0, B, _, A, '#'),
+    !,
+    sub_atom(Ref0, 0, B, _, File),
+    sub_atom(Ref0, _, A, 0, Fragment),
+    file_directory_name(Path, Dir),
+    atomic_list_concat([Dir, /, File], SecPath),
+    Obj = section(_, _, _, SecPath),
+    manual_object(Obj, _, _, _, _),
+    !,
+    object_href(Obj, Ref1),
+    format(string(Ref), '~w#~w', [Ref1, Fragment]).
 
 %!  referenced_section(+Fragment, +File, +Path, -Section)
 
