@@ -44,7 +44,8 @@
 :- use_module(library(dialect/xsb/increval)).
 
 test_monotonic :-
-    run_tests([ monotonic_tabling
+    run_tests([ monotonic_tabling,
+                monotonic_tabling_2
               ]).
 
 :- begin_tests(monotonic_tabling).
@@ -62,7 +63,9 @@ cleanup :-
     retractall(db(_)),
     retractall(dx(_)),
     retractall(lm(_,_)),
-    retractall(la(_,_)).
+    retractall(la(_,_)),
+    retractall(nd1(_)),
+    retractall(nd2(_)).
 
 % Very basic test
 
@@ -146,18 +149,6 @@ test(closure2, PairsMS == PairsAS) :-
     sort(PairsM, PairsMS),
     sort(PairsA, PairsAS).
 
-% Test dependency error
-
-:- dynamic de/1 as monotonic.
-:- table pe/1.
-
-pe(X) :- de(Y), X is 7*Y.
-
-test(dep_error,
-     error(dependency_error(plunit_monotonic_tabling:pe(_),
-                            monotonic(plunit_monotonic_tabling:de(_))))) :-
-    pe(_).
-
 % Incremental tables can depend on monotonic tables
 
 :- dynamic dx/1 as (incremental,monotonic).
@@ -177,4 +168,87 @@ test(incr_mono, true) :-
     forall(qx(_), true),
     assertion(get_returns_for_call(qx(_), qx(6))).
 
+% Monotonic table can have both monotonic and incremental dependents
+
+:- table pmi/1 as monotonic.
+:- dynamic dm/1 as monotonic.
+:- dynamic di/1 as incremental.
+
+pmi(X) :-
+    dm(X),
+    \+ di(X).
+
+test(incr_mono, L == [1]) :-
+    forall(pmi(_), true),
+    asserta(dm(1)),
+    asserta(dm(2)),
+    asserta(di(2)),
+    assertion(incr_is_invalid(pmi(_))),
+    findall(X, pmi(X), L).
+
+% Test nested reevaluation of a monotonic table that was invalidated
+% due to a retract
+
+:- dynamic (nd1/1, nd2/1) as (incremental,monotonic).
+:- table (np/1,ni/1) as (incremental).
+:- table (nm/1) as monotonic.
+
+np(X) :-
+    (   ni(X)
+    ;   nm(X)
+    ).
+
+ni(X) :- nd1(X).
+nm(X) :- nd2(X).
+
+test(nested_reeval_mono, true) :-
+    assert(nd1(1)),
+    assert(nd2(2)),
+    setof(X, np(X), Xs1),
+    assertion(Xs1 == [1,2]),
+    assert(nd1(3)),
+    retract(nd2(2)),
+    assert(nd2(3)),
+    setof(X, np(X), Xs2),
+    assertion(Xs2 == [1,3]).
+
 :- end_tests(monotonic_tabling).
+
+% Monotonic table that is invalidated due to retract and is
+% re-evaluated twice.
+
+:- begin_tests(monotonic_tabling_2).
+
+:- dynamic (d1/1, d2/1, d3/1) as (incremental,monotonic).
+:- table (p/1,i/1) as (incremental).
+:- table (m/1,n/1) as (monotonic,lazy).
+
+p(X) :-
+    (   i(X)
+    ;   m(X)
+    ).
+
+i(X) :- d3(X).
+m(X) :- n(X).
+m(X) :- d2(X).
+
+n(X) :- d1(X).
+
+clean :-
+    retractall(d1(_)),
+    retractall(d2(_)),
+    retractall(d3(_)),
+    abolish_all_tables.
+
+test(twice_invalid, [Xs == [1,2,3], cleanup(clean)]) :-
+    assert(d1(1)),
+    assert(d2(2)),
+    forall(p(_), true),
+    retract(d1(1)),
+    assert(d1(1)),
+    retract(d2(2)),
+    assert(d2(2)),
+    assert(d3(3)),
+    setof(X, p(X), Xs).
+
+:- end_tests(monotonic_tabling_2).
