@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2020, University of Amsterdam
+    Copyright (c)  1985-2021, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -477,7 +478,10 @@ push_goal(LocalFrame fr)
 	    *ad++ = ATOM_nil;
 	  } else
 	  { *ad++ = consInt(len);
-	    *ad++ = *tail;
+	    if ( isTerm(*tail) )
+	      *ad++ = ATOM_compound;
+	    else
+	      *ad++ = *tail;
 	  }
 	} else
 	{ FunctorDef f = valueFunctor(functorTerm(*a));
@@ -595,7 +599,7 @@ push_overflow_context(Stack stack, int extra)
 
     *dict = dict_functor((p-dict-2)/2);		/* final functor */
 
-    dict_order(dict, FALSE PASS_LD);
+    dict_order(dict, NULL PASS_LD);
 
     return consPtr(dict, STG_GLOBAL|TAG_COMPOUND);
   } else
